"""Autogenerated API"""
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module

log = logging.getLogger(__name__)


@register_command(
    extending=("alarms","v1","signature"),
    module=argus_cli_module
)
def delete_signatures(
    signature: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Delete signatures. They must not be mapped to an alarm. (INTERNAL)
    
    :param list signature: Signatures to delete
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import delete
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/alarms/v1/signature".format(settings["api"]["api_url"], signature=signature)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send signature if the argument was provided, dont send null values
    if signature is not None:
        query_parameters.update({ "signature": signature })

    log.debug("DELETE %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = delete(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("alarms","v1","signature"),
    module=argus_cli_module
)
def get_signatures(
    keywords: str = None,
    keywordField: str = None,
    keywordMatch: str = "all",
    limit: int = 25,
    offset: int = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Get all signatures including alarms if mapped (PUBLIC)
    
    :param list keywords: Search by keywords
    :param list keywordField: Set field strategy for keyword search
    :param str keywordMatch: Set match strategy for keyword search
    :param int limit: Maximum number of returned signatures
    :param int offset: Skip a number of signatures
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/alarms/v1/signature".format(settings["api"]["api_url"], keywordMatch=keywordMatch, limit=limit, keywords=keywords, keywordField=keywordField, offset=offset)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send keywordMatch if the argument was provided, dont send null values
    if keywordMatch is not None:
        query_parameters.update({ "keywordMatch": keywordMatch })
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        query_parameters.update({ "limit": limit })
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        query_parameters.update({ "keywords": keywords })
    # Only send keywordField if the argument was provided, dont send null values
    if keywordField is not None:
        query_parameters.update({ "keywordField": keywordField })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        query_parameters.update({ "offset": offset })

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("alarms","v1","signature"),
    module=argus_cli_module
)
def search_signatures(
    limit: int = None,
    offset: int = None,
    includeDeleted: bool = None,
    includeFlags: int = None,
    excludeFlags: int = None,
    subCriteria: dict = None,
    exclude: bool = None,
    required: bool = None,
    attackCategoryID: int = None,
    alarmID: int = None,
    signature: str = None,
    minTriggerAmount: int = None,
    maxTriggerAmount: int = None,
    startTimestamp: int = None,
    endTimestamp: int = None,
    timeFieldStrategy: str = None,
    timeMatchStrategy: str = None,
    keywords: str = None,
    keywordFieldStrategy: str = None,
    keywordMatchStrategy: str = None,
    sortBy: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Get all signatures matching a given search criteria (PUBLIC)
    
    :param int limit: Set this value to set max number of results. By default, no restriction on result set size. 
    :param int offset: Set this value to skip the first (offset) objects. By default, return result from first object. 
    :param bool includeDeleted: Set to true to include deleted objects. By default, exclude deleted objects. 
    :param int includeFlags: Only include objects which have includeFlags set. 
    :param int excludeFlags: Exclude objects which have excludeFlags set. 
    :param list subCriteria: Set additional criterias which are applied using a logical OR. 
    :param bool exclude: Only relevant for subcriteria. If set to true, objects matching this subcriteria object will be excluded. 
    :param bool required: Only relevant for subcriteria. If set to true, objects matching this subcriteria are required (AND-ed together with parent criteria). 
    :param list attackCategoryID: A set of IDs for attack categories (alarm category). 
    :param list alarmID: A set of IDs for alarms. 
    :param list signature: A set of signatures. It does an exact match. 
    :param int minTriggerAmount: Minimum trigger amount, default 0 means disabled 
    :param int maxTriggerAmount: Maximum trigger amount, default 0 means disabled 
    :param int startTimestamp: Only include mappings based on the set TimeFieldStrategy and TimeMatchStrategy (start timestamp) 
    :param int endTimestamp: Only include mappings based on the set TimeFieldStrategy and TimeMatchStrategy (end timestamp) 
    :param list timeFieldStrategy: TimeFieldStrategy to define which timestamp field(s) to match. (default lastTriggeredTimestamp)
    :param str timeMatchStrategy: TimeMatchStrategy to define how to match startTimestamp and endTimestamp with fields. (default any)
    :param list keywords: A set of keywords matched against mappings based on the set KeywordFieldStrategy and KeywordMatchStrategy. 
    :param list keywordFieldStrategy: KeywordFieldStrategy to define which field(s) to match against keywords. (default all)
    :param str keywordMatchStrategy: KeywordMatchStrategy to define how to match keywords with fields. (default all)
    :param list sortBy: List of properties to sort by (prefix with "-" to sort descending). 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import post
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/alarms/v1/signature/search".format(settings["api"]["api_url"], )
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        body.update({ "limit": limit })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        body.update({ "offset": offset })
    # Only send includeDeleted if the argument was provided, dont send null values
    if includeDeleted is not None:
        body.update({ "includeDeleted": includeDeleted })
    # Only send includeFlags if the argument was provided, dont send null values
    if includeFlags is not None:
        body.update({ "includeFlags": includeFlags })
    # Only send excludeFlags if the argument was provided, dont send null values
    if excludeFlags is not None:
        body.update({ "excludeFlags": excludeFlags })
    # Only send subCriteria if the argument was provided, dont send null values
    if subCriteria is not None:
        body.update({ "subCriteria": subCriteria })
    # Only send exclude if the argument was provided, dont send null values
    if exclude is not None:
        body.update({ "exclude": exclude })
    # Only send required if the argument was provided, dont send null values
    if required is not None:
        body.update({ "required": required })
    # Only send attackCategoryID if the argument was provided, dont send null values
    if attackCategoryID is not None:
        body.update({ "attackCategoryID": attackCategoryID })
    # Only send alarmID if the argument was provided, dont send null values
    if alarmID is not None:
        body.update({ "alarmID": alarmID })
    # Only send signature if the argument was provided, dont send null values
    if signature is not None:
        body.update({ "signature": signature })
    # Only send minTriggerAmount if the argument was provided, dont send null values
    if minTriggerAmount is not None:
        body.update({ "minTriggerAmount": minTriggerAmount })
    # Only send maxTriggerAmount if the argument was provided, dont send null values
    if maxTriggerAmount is not None:
        body.update({ "maxTriggerAmount": maxTriggerAmount })
    # Only send startTimestamp if the argument was provided, dont send null values
    if startTimestamp is not None:
        body.update({ "startTimestamp": startTimestamp })
    # Only send endTimestamp if the argument was provided, dont send null values
    if endTimestamp is not None:
        body.update({ "endTimestamp": endTimestamp })
    # Only send timeFieldStrategy if the argument was provided, dont send null values
    if timeFieldStrategy is not None:
        body.update({ "timeFieldStrategy": timeFieldStrategy })
    # Only send timeMatchStrategy if the argument was provided, dont send null values
    if timeMatchStrategy is not None:
        body.update({ "timeMatchStrategy": timeMatchStrategy })
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        body.update({ "keywords": keywords })
    # Only send keywordFieldStrategy if the argument was provided, dont send null values
    if keywordFieldStrategy is not None:
        body.update({ "keywordFieldStrategy": keywordFieldStrategy })
    # Only send keywordMatchStrategy if the argument was provided, dont send null values
    if keywordMatchStrategy is not None:
        body.update({ "keywordMatchStrategy": keywordMatchStrategy })
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        body.update({ "sortBy": sortBy })

    query_parameters = {}

    log.debug("POST %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = post(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()

