"""Autogenerated API"""
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module

log = logging.getLogger(__name__)


@register_command(
    extending=("assets","v1","scan"),
    module=argus_cli_module
)
def client_host_asset_scanned(
    customerID: int,
    hostName: str,
    detectedVulnerabilities: dict = None,
    detectedApplications: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Marks a single Client HostAsset as scanned. The host is identified by the host name, and it must be of type CLIENT. (DEV)
    
    :param int customerID: Customer ID
    :param str hostName: Name of host asset
    :param list detectedVulnerabilities: Specify (vulnerabilityID, socket string) objects of detected vulnerabilities. 
    :param list detectedApplications: Specify sockets of detected applications (e.g. tcp/80). 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v1/scan/{customerID}/client/{hostName}".format(settings["api"]["api_url"], customerID=customerID, hostName=hostName)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send detectedVulnerabilities if the argument was provided, dont send null values
    if detectedVulnerabilities is not None:
        body.update({ "detectedVulnerabilities": detectedVulnerabilities })
    # Only send detectedApplications if the argument was provided, dont send null values
    if detectedApplications is not None:
        body.update({ "detectedApplications": detectedApplications })

    query_parameters = {}

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v1","scan"),
    module=argus_cli_module
)
def host_asset_bulk_scanned(
    customerID: int = None,
    scannedRequests: dict = None,
    scannedIpRanges: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Marks multiple HostAsset as scanned. (PUBLIC)
    
    :param int customerID: Define customer which was scanned. 
    :param list scannedRequests: Set of host scan requests. 
    :param list scannedIpRanges: Set of scanned IP address ranges as list of single IPs (1.1.1.1), CIDR networks (1.1.1.0/24) or ranges (1.1.1.1-1.1.1.2) 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import post
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v1/scan".format(settings["api"]["api_url"], )
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send customerID if the argument was provided, dont send null values
    if customerID is not None:
        body.update({ "customerID": customerID })
    # Only send scannedRequests if the argument was provided, dont send null values
    if scannedRequests is not None:
        body.update({ "scannedRequests": scannedRequests })
    # Only send scannedIpRanges if the argument was provided, dont send null values
    if scannedIpRanges is not None:
        body.update({ "scannedIpRanges": scannedIpRanges })

    query_parameters = {}

    log.debug("POST %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = post(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v1","scan"),
    module=argus_cli_module
)
def host_asset_scanned(
    customerID: int,
    ip: str,
    detectedVulnerabilities: dict = None,
    detectedApplications: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Marks a single HostAsset as scanned. (PUBLIC)
    
    :param int customerID: Customer ID
    :param str ip: IP address of scanned HostAsset
    :param list detectedVulnerabilities: Specify (vulnerabilityID, socket string) objects of detected vulnerabilities. 
    :param list detectedApplications: Specify sockets of detected applications (e.g. tcp/80). 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v1/scan/{customerID}/{ip}".format(settings["api"]["api_url"], customerID=customerID, ip=ip)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send detectedVulnerabilities if the argument was provided, dont send null values
    if detectedVulnerabilities is not None:
        body.update({ "detectedVulnerabilities": detectedVulnerabilities })
    # Only send detectedApplications if the argument was provided, dont send null values
    if detectedApplications is not None:
        body.update({ "detectedApplications": detectedApplications })

    query_parameters = {}

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v1","scan"),
    module=argus_cli_module
)
def host_asset_scanned_by_id(
    hostID: str,
    detectedVulnerabilities: dict = None,
    detectedApplications: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Marks a single HostAsset as scanned. The host is identified by ID. (DEV)
    
    :param str hostID: ID of the host asset
    :param list detectedVulnerabilities: Specify (vulnerabilityID, socket string) objects of detected vulnerabilities. 
    :param list detectedApplications: Specify sockets of detected applications (e.g. tcp/80). 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v1/scan/{hostID}".format(settings["api"]["api_url"], hostID=hostID)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send detectedVulnerabilities if the argument was provided, dont send null values
    if detectedVulnerabilities is not None:
        body.update({ "detectedVulnerabilities": detectedVulnerabilities })
    # Only send detectedApplications if the argument was provided, dont send null values
    if detectedApplications is not None:
        body.update({ "detectedApplications": detectedApplications })

    query_parameters = {}

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()

