"""Autogenerated API"""
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module

log = logging.getLogger(__name__)


@register_command(
    extending=("componentadmin","v2","instance","status"),
    module=argus_cli_module
)
def update_instance_status2(
    id: int,
    containerID: str = None,
    identifiers: dict = None,
    state: str = None,
    components: dict = None,
    instanceID: int = None,
    runningInstanceRevision: int = None,
    runningTemplateRevision: int = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """null (DEV)
    
    :param int id: ID of instance
    :param str containerID: ID of the currently running container 
    :param dict identifiers: Any identifiers for this instance container 
    :param str state: The state of this instance container 
    :param list components: An optional list of component updates for the root container 
    :param int instanceID: The instanceID of this container 
    :param int runningInstanceRevision: The instance revision of the running container 
    :param int runningTemplateRevision: The template revision of the running container 
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/componentadmin/v2/instance/{id}/status".format(settings["api"]["api_url"], id=id)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send containerID if the argument was provided, dont send null values
    if containerID is not None:
        body.update({ "containerID": containerID })
    # Only send identifiers if the argument was provided, dont send null values
    if identifiers is not None:
        body.update({ "identifiers": identifiers })
    # Only send state if the argument was provided, dont send null values
    if state is not None:
        body.update({ "state": state })
    # Only send components if the argument was provided, dont send null values
    if components is not None:
        body.update({ "components": components })
    # Only send instanceID if the argument was provided, dont send null values
    if instanceID is not None:
        body.update({ "instanceID": instanceID })
    # Only send runningInstanceRevision if the argument was provided, dont send null values
    if runningInstanceRevision is not None:
        body.update({ "runningInstanceRevision": runningInstanceRevision })
    # Only send runningTemplateRevision if the argument was provided, dont send null values
    if runningTemplateRevision is not None:
        body.update({ "runningTemplateRevision": runningTemplateRevision })

    query_parameters = {}

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()

