"""Autogenerated API"""
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module
from requests import Response

log = logging.getLogger(__name__)


@register_command(
    extending=("currentuser","v1","groups"),
    module=argus_cli_module
)
def get_current_user_groups(
    limit: int = 25,
    includeAncestors: bool = None,
    offset: int = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> Response:
    """Get list of groups which current user is a member of. Result will be sorted by id (ascending) (PUBLIC)
    
    :param int limit: Maximum number of values to return
    :param bool includeAncestors: If true, also include group ancestors
    :param int offset: Skip this number of records
    :raises AuthenticationFailedException: on 401:raises ArgusException: on other status codes
    
    :returns: requests.Response object
    

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/currentuser/v1/groups".format(settings["api"]["api_url"], limit=limit, includeAncestors=includeAncestors, offset=offset)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        query_parameters.update({ "limit": limit })
    # Only send includeAncestors if the argument was provided, dont send null values
    if includeAncestors is not None:
        query_parameters.update({ "includeAncestors": includeAncestors })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        query_parameters.update({ "offset": offset })

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response
    

