"""Autogenerated API"""
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module

log = logging.getLogger(__name__)


@register_command(
    extending=("pdns","v3","search"),
    module=argus_cli_module
)
def search_records(
    query: str = None,
    aggregateResult: bool = None,
    includeAnonymousResults: bool = None,
    rrClass: str = None,
    rrType: str = None,
    customerID: int = None,
    tlp: str = None,
    limit: int = None,
    offset: int = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Search against PassiveDNS with criteria and return matching records (DEV)
    
    :param str query: Lookup query 
    :param bool aggregateResult: Whether aggregate results (default true) 
    :param bool includeAnonymousResults: Whether include anonymous results (default true) 
    :param list rrClass: Lookup with specified record classes 
    :param list rrType: Lookup with specified record types 
    :param list customerID: Lookup for specified customer IDs 
    :param list tlp: Lookup with specified TLPs, public usage only TLP white allowed 
    :param int limit: Max number of results to be returned, default unset means default limit 25 will be used, 0 means unlimited 
    :param int offset: Number of results to be skipped first, default 0 (default 0)
    :raises AuthenticationFailedException: on 401
    :raises ResourceLimitExceeded.Exception: on 402
    :raises AccessDeniedException: on 403
    :raises ValidationFailedException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import post
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/pdns/v3/search".format(settings["api"]["api_url"], )
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send query if the argument was provided, dont send null values
    if query is not None:
        body.update({ "query": query })
    # Only send aggregateResult if the argument was provided, dont send null values
    if aggregateResult is not None:
        body.update({ "aggregateResult": aggregateResult })
    # Only send includeAnonymousResults if the argument was provided, dont send null values
    if includeAnonymousResults is not None:
        body.update({ "includeAnonymousResults": includeAnonymousResults })
    # Only send rrClass if the argument was provided, dont send null values
    if rrClass is not None:
        body.update({ "rrClass": rrClass })
    # Only send rrType if the argument was provided, dont send null values
    if rrType is not None:
        body.update({ "rrType": rrType })
    # Only send customerID if the argument was provided, dont send null values
    if customerID is not None:
        body.update({ "customerID": customerID })
    # Only send tlp if the argument was provided, dont send null values
    if tlp is not None:
        body.update({ "tlp": tlp })
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        body.update({ "limit": limit })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        body.update({ "offset": offset })

    query_parameters = {}

    log.debug("POST %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = post(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()

