"""Autogenerated API"""
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module

log = logging.getLogger(__name__)


@register_command(
    extending=("useradmin","v2","function"),
    module=argus_cli_module
)
def add_function(
    name: str = None,
    parent: str = None,
    description: str = None,
    minimumSecurityLevel: str = None,
    role: bool = None,
    sysadm: bool = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Add new function with a parent function (PUBLIC)
    
    :param str name: The name of the function  => [a-zA-Z0-9_\-\.]*
    :param str parent: The functions parent (id or name) 
    :param str description: Describes what the function does  => [\s\w\{\}\$\-\(\)\.\[\]"\'_/\\,\*\+\#:@!?;=]*
    :param str minimumSecurityLevel: The minimum security level of the function. (default standard)
    :param bool role: Can this function be assigned to a user? (default false)
    :param bool sysadm: Mark this function as a sysadm function? (default false)
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import post
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/useradmin/v2/function".format(settings["api"]["api_url"], )
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({ "name": name })
    # Only send parent if the argument was provided, dont send null values
    if parent is not None:
        body.update({ "parent": parent })
    # Only send description if the argument was provided, dont send null values
    if description is not None:
        body.update({ "description": description })
    # Only send minimumSecurityLevel if the argument was provided, dont send null values
    if minimumSecurityLevel is not None:
        body.update({ "minimumSecurityLevel": minimumSecurityLevel })
    # Only send role if the argument was provided, dont send null values
    if role is not None:
        body.update({ "role": role })
    # Only send sysadm if the argument was provided, dont send null values
    if sysadm is not None:
        body.update({ "sysadm": sysadm })

    query_parameters = {}

    log.debug("POST %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = post(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("useradmin","v2","function"),
    module=argus_cli_module
)
def add_function_child(
    functionNameOrID: str,
    children: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Adds functions to a parent function (PUBLIC)
    
    :param str functionNameOrID: The name or id of the function
    :param list children: The children to add to the function (id or name) 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import post
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/useradmin/v2/function/{functionNameOrID}/children".format(settings["api"]["api_url"], functionNameOrID=functionNameOrID)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send children if the argument was provided, dont send null values
    if children is not None:
        body.update({ "children": children })

    query_parameters = {}

    log.debug("POST %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = post(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("useradmin","v2","function"),
    module=argus_cli_module
)
def delete_user(
    functionNameOrID: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Deletes a function (PUBLIC)
    
    :param str functionNameOrID: The name or id of the function
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import delete
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/useradmin/v2/function/{functionNameOrID}".format(settings["api"]["api_url"], functionNameOrID=functionNameOrID)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("DELETE %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = delete(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("useradmin","v2","function"),
    module=argus_cli_module
)
def get_function(
    functionNameOrID: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Get a function (PUBLIC)
    
    :param str functionNameOrID: The name or id of the function
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/useradmin/v2/function/{functionNameOrID}".format(settings["api"]["api_url"], functionNameOrID=functionNameOrID)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("useradmin","v2","function"),
    module=argus_cli_module
)
def list_function_children(
    functionNameOrID: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Returns the children of a function (PUBLIC)
    
    :param str functionNameOrID: The name or id of the function
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/useradmin/v2/function/{functionNameOrID}/children".format(settings["api"]["api_url"], functionNameOrID=functionNameOrID)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("useradmin","v2","function"),
    module=argus_cli_module
)
def list_function_parents(
    functionNameOrID: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Returns the parents of a function (PUBLIC)
    
    :param str functionNameOrID: The name or id of the function
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/useradmin/v2/function/{functionNameOrID}/parents".format(settings["api"]["api_url"], functionNameOrID=functionNameOrID)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("useradmin","v2","function"),
    module=argus_cli_module
)
def list_functions(
    function: str = None,
    minimumSecurityLevel: str = None,
    excludeFlag: str = None,
    includeFlag: str = None,
    ancestor: str = None,
    descendant: str = None,
    keyword: str = None,
    keywordField: str = None,
    sortBy: str = None,
    limit: int = 25,
    keywordMatch: str = "all",
    offset: int = None,
    includeDeleted: bool = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Returns the functions matching the query (PUBLIC)
    
    :param list function: Search by function name or ID
    :param str minimumSecurityLevel: Specifies the minimum security level of functions to be returned
    :param list excludeFlag: Exclude functions with flag
    :param list includeFlag: Include functions with flag
    :param list ancestor: Search in ancestors of function
    :param list descendant: Search in descendants of function
    :param list keyword: Search by keywords
    :param list keywordField: Set field strategy for keyword search
    :param list sortBy: Sort search result
    :param int limit: Maximum number of returned results
    :param str keywordMatch: Set match strategy for keyword search
    :param int offset: Skip a number of results
    :param bool includeDeleted: Include deleted functions
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/useradmin/v2/function".format(settings["api"]["api_url"], limit=limit, keywordMatch=keywordMatch, offset=offset, function=function, minimumSecurityLevel=minimumSecurityLevel, includeDeleted=includeDeleted, excludeFlag=excludeFlag, includeFlag=includeFlag, ancestor=ancestor, descendant=descendant, keyword=keyword, keywordField=keywordField, sortBy=sortBy)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        query_parameters.update({ "limit": limit })
    # Only send keywordMatch if the argument was provided, dont send null values
    if keywordMatch is not None:
        query_parameters.update({ "keywordMatch": keywordMatch })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        query_parameters.update({ "offset": offset })
    # Only send function if the argument was provided, dont send null values
    if function is not None:
        query_parameters.update({ "function": function })
    # Only send minimumSecurityLevel if the argument was provided, dont send null values
    if minimumSecurityLevel is not None:
        query_parameters.update({ "minimumSecurityLevel": minimumSecurityLevel })
    # Only send includeDeleted if the argument was provided, dont send null values
    if includeDeleted is not None:
        query_parameters.update({ "includeDeleted": includeDeleted })
    # Only send excludeFlag if the argument was provided, dont send null values
    if excludeFlag is not None:
        query_parameters.update({ "excludeFlag": excludeFlag })
    # Only send includeFlag if the argument was provided, dont send null values
    if includeFlag is not None:
        query_parameters.update({ "includeFlag": includeFlag })
    # Only send ancestor if the argument was provided, dont send null values
    if ancestor is not None:
        query_parameters.update({ "ancestor": ancestor })
    # Only send descendant if the argument was provided, dont send null values
    if descendant is not None:
        query_parameters.update({ "descendant": descendant })
    # Only send keyword if the argument was provided, dont send null values
    if keyword is not None:
        query_parameters.update({ "keyword": keyword })
    # Only send keywordField if the argument was provided, dont send null values
    if keywordField is not None:
        query_parameters.update({ "keywordField": keywordField })
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        query_parameters.update({ "sortBy": sortBy })

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("useradmin","v2","function"),
    module=argus_cli_module
)
def remove_function_child(
    functionNameOrID: str,
    child: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Removes functions from a parent function (PUBLIC)
    
    :param str functionNameOrID: The name or id of the function
    :param list child: Name or ID of child functions to remove
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import delete
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/useradmin/v2/function/{functionNameOrID}/children".format(settings["api"]["api_url"], functionNameOrID=functionNameOrID, child=child)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send child if the argument was provided, dont send null values
    if child is not None:
        query_parameters.update({ "child": child })

    log.debug("DELETE %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = delete(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("useradmin","v2","function"),
    module=argus_cli_module
)
def search_functions(
    function: str = None,
    ancestors: str = None,
    descendants: str = None,
    minimumSecurityLevel: str = None,
    includeFlags: str = None,
    excludeFlags: str = None,
    subCriteria: dict = None,
    keywords: str = None,
    keywordFieldStrategy: str = None,
    keywordMatchStrategy: str = None,
    sortBy: str = None,
    includeDeleted: bool = None,
    limit: int = 25,
    offset: int = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Returns the functions matching the query (PUBLIC)
    
    :param list function: Function to search for (name or id) 
    :param list ancestors: Specifies the function ancestors to search in. Can be the name or id 
    :param list descendants: Specifies the function descendants to search in. Can be the name or id 
    :param str minimumSecurityLevel: Specifies the minimum security level of functions to be returned 
    :param list includeFlags: Flags to include 
    :param list excludeFlags: Flags to exclude 
    :param list subCriteria: Subcriteria to add to the search 
    :param list keywords: Keywords to search for 
    :param list keywordFieldStrategy: Which fields will be searched for the given keyword (default All supported fields)
    :param str keywordMatchStrategy: Search based on all keywords (AND), or based on any keyword (OR) (default Match all keywords (AND))
    :param list sortBy: Field to sort result by (default name)
    :param bool includeDeleted: Include deleted functions? (default false)
    :param int limit: The max amount of items to display (default 25)
    :param int offset: The amount of items to skip (default 0)
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import post
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/useradmin/v2/function/search".format(settings["api"]["api_url"], )
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send function if the argument was provided, dont send null values
    if function is not None:
        body.update({ "function": function })
    # Only send ancestors if the argument was provided, dont send null values
    if ancestors is not None:
        body.update({ "ancestors": ancestors })
    # Only send descendants if the argument was provided, dont send null values
    if descendants is not None:
        body.update({ "descendants": descendants })
    # Only send minimumSecurityLevel if the argument was provided, dont send null values
    if minimumSecurityLevel is not None:
        body.update({ "minimumSecurityLevel": minimumSecurityLevel })
    # Only send includeDeleted if the argument was provided, dont send null values
    if includeDeleted is not None:
        body.update({ "includeDeleted": includeDeleted })
    # Only send includeFlags if the argument was provided, dont send null values
    if includeFlags is not None:
        body.update({ "includeFlags": includeFlags })
    # Only send excludeFlags if the argument was provided, dont send null values
    if excludeFlags is not None:
        body.update({ "excludeFlags": excludeFlags })
    # Only send subCriteria if the argument was provided, dont send null values
    if subCriteria is not None:
        body.update({ "subCriteria": subCriteria })
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        body.update({ "keywords": keywords })
    # Only send keywordFieldStrategy if the argument was provided, dont send null values
    if keywordFieldStrategy is not None:
        body.update({ "keywordFieldStrategy": keywordFieldStrategy })
    # Only send keywordMatchStrategy if the argument was provided, dont send null values
    if keywordMatchStrategy is not None:
        body.update({ "keywordMatchStrategy": keywordMatchStrategy })
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        body.update({ "sortBy": sortBy })
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        body.update({ "limit": limit })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        body.update({ "offset": offset })

    query_parameters = {}

    log.debug("POST %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = post(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("useradmin","v2","function"),
    module=argus_cli_module
)
def update_function(
    functionNameOrID: str,
    name: str = None,
    description: str = None,
    minimumSecurityLevel: str = None,
    role: bool = None,
    sysadm: bool = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Update a function (PUBLIC)
    
    :param str functionNameOrID: The name or id of the function
    :param str name: The new name of the function  => [a-zA-Z0-9_\-\.]*
    :param str description: Describes what the function does  => [\s\w\{\}\$\-\(\)\.\[\]"\'_/\\,\*\+\#:@!?;=]*
    :param str minimumSecurityLevel: The new minimum security level of the function. 
    :param bool role: Can this function be assigned to a user? 
    :param bool sysadm: Mark this function as a sysadm function? 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/useradmin/v2/function/{functionNameOrID}".format(settings["api"]["api_url"], functionNameOrID=functionNameOrID)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({ "name": name })
    # Only send description if the argument was provided, dont send null values
    if description is not None:
        body.update({ "description": description })
    # Only send minimumSecurityLevel if the argument was provided, dont send null values
    if minimumSecurityLevel is not None:
        body.update({ "minimumSecurityLevel": minimumSecurityLevel })
    # Only send role if the argument was provided, dont send null values
    if role is not None:
        body.update({ "role": role })
    # Only send sysadm if the argument was provided, dont send null values
    if sysadm is not None:
        body.update({ "sysadm": sysadm })

    query_parameters = {}

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()

