"""Autogenerated API"""
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module
from requests import Response

log = logging.getLogger(__name__)


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def check_permission(
    userId: int,
    function: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> Response:
    """Validate user permission to perform function (PUBLIC)
    
    :param int userId: ID of user
    :param str function: Function name
    :raises AuthenticationFailedException: on 401
    :raises ForbiddenPermissionException: on 403
    :raises UserNotFoundException: on 404
    :raises ValidationFailedException: on 412:raises ArgusException: on other status codes
    
    :returns: requests.Response object
    

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user/{userId}/permission/{function}".format(settings["api"]["api_url"], userId=userId, function=function)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response
    


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def find_users(
    domain: str = None,
    type: str = None,
    subjectID: int = None,
    excludeSubjectID: int = None,
    parentID: int = None,
    ancestorID: int = None,
    get_aclAncestorID: int = None,
    get_aclCustomerID: int = None,
    excludeAncestorID: int = None,
    userName: str = None,
    email: str = None,
    mobile: str = None,
    searchString: str = None,
    requiredFunctionID: int = None,
    requiredFunctionCustomerID: int = None,
    requireAllFunctions: bool = None,
    limit: int = None,
    offset: int = None,
    includeDeleted: bool = None,
    includeFlags: int = None,
    excludeFlags: int = None,
    subCriteria: dict = None,
    exclude: bool = None,
    required: bool = None,
    customerID: int = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Search user profiles (PUBLIC)
    
    :param list domain: 
    :param str type: 
    :param list subjectID: 
    :param list excludeSubjectID: 
    :param list parentID: 
    :param list ancestorID: 
    :param list get_aclAncestorID: 
    :param list get_aclCustomerID: 
    :param list excludeAncestorID: 
    :param list userName: 
    :param list email: 
    :param list mobile: 
    :param str searchString: 
    :param list requiredFunctionID: 
    :param int requiredFunctionCustomerID: 
    :param bool requireAllFunctions: 
    :param int limit: Set this value to set max number of results. By default, no restriction on result set size. 
    :param int offset: Set this value to skip the first (offset) objects. By default, return result from first object. 
    :param bool includeDeleted: Set to true to include deleted objects. By default, exclude deleted objects. 
    :param int includeFlags: Only include objects which have includeFlags set. 
    :param int excludeFlags: Exclude objects which have excludeFlags set. 
    :param list subCriteria: Set additional criterias which are applied using a logical OR. 
    :param bool exclude: Only relevant for subcriteria. If set to true, objects matching this subcriteria object will be excluded. 
    :param bool required: Only relevant for subcriteria. If set to true, objects matching this subcriteria are required (AND-ed together with parent criteria). 
    :param list customerID: Restrict search to data belonging to specified customers. 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import post
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user/search".format(settings["api"]["api_url"], )
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send domain if the argument was provided, dont send null values
    if domain is not None:
        body.update({ "domain": domain })
    # Only send type if the argument was provided, dont send null values
    if type is not None:
        body.update({ "type": type })
    # Only send subjectID if the argument was provided, dont send null values
    if subjectID is not None:
        body.update({ "subjectID": subjectID })
    # Only send excludeSubjectID if the argument was provided, dont send null values
    if excludeSubjectID is not None:
        body.update({ "excludeSubjectID": excludeSubjectID })
    # Only send parentID if the argument was provided, dont send null values
    if parentID is not None:
        body.update({ "parentID": parentID })
    # Only send ancestorID if the argument was provided, dont send null values
    if ancestorID is not None:
        body.update({ "ancestorID": ancestorID })
    # Only send get_aclAncestorID if the argument was provided, dont send null values
    if get_aclAncestorID is not None:
        body.update({ "get_aclAncestorID": get_aclAncestorID })
    # Only send get_aclCustomerID if the argument was provided, dont send null values
    if get_aclCustomerID is not None:
        body.update({ "get_aclCustomerID": get_aclCustomerID })
    # Only send excludeAncestorID if the argument was provided, dont send null values
    if excludeAncestorID is not None:
        body.update({ "excludeAncestorID": excludeAncestorID })
    # Only send userName if the argument was provided, dont send null values
    if userName is not None:
        body.update({ "userName": userName })
    # Only send email if the argument was provided, dont send null values
    if email is not None:
        body.update({ "email": email })
    # Only send mobile if the argument was provided, dont send null values
    if mobile is not None:
        body.update({ "mobile": mobile })
    # Only send searchString if the argument was provided, dont send null values
    if searchString is not None:
        body.update({ "searchString": searchString })
    # Only send requiredFunctionID if the argument was provided, dont send null values
    if requiredFunctionID is not None:
        body.update({ "requiredFunctionID": requiredFunctionID })
    # Only send requiredFunctionCustomerID if the argument was provided, dont send null values
    if requiredFunctionCustomerID is not None:
        body.update({ "requiredFunctionCustomerID": requiredFunctionCustomerID })
    # Only send requireAllFunctions if the argument was provided, dont send null values
    if requireAllFunctions is not None:
        body.update({ "requireAllFunctions": requireAllFunctions })
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        body.update({ "limit": limit })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        body.update({ "offset": offset })
    # Only send includeDeleted if the argument was provided, dont send null values
    if includeDeleted is not None:
        body.update({ "includeDeleted": includeDeleted })
    # Only send includeFlags if the argument was provided, dont send null values
    if includeFlags is not None:
        body.update({ "includeFlags": includeFlags })
    # Only send excludeFlags if the argument was provided, dont send null values
    if excludeFlags is not None:
        body.update({ "excludeFlags": excludeFlags })
    # Only send subCriteria if the argument was provided, dont send null values
    if subCriteria is not None:
        body.update({ "subCriteria": subCriteria })
    # Only send exclude if the argument was provided, dont send null values
    if exclude is not None:
        body.update({ "exclude": exclude })
    # Only send required if the argument was provided, dont send null values
    if required is not None:
        body.update({ "required": required })
    # Only send customerID if the argument was provided, dont send null values
    if customerID is not None:
        body.update({ "customerID": customerID })

    query_parameters = {}

    log.debug("POST %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = post(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def get_functions(
    userId: int,
    onlyRoles: bool = None,
    keywords: str = None,
    limit: int = 25,
    offset: int = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> Response:
    """List a user's permission functions regardless of customer (PUBLIC)
    
    :param int userId: ID of user
    :param bool onlyRoles: Only return functions marked as roles
    :param list keywords: Filter functions by keywords on name and description
    :param int limit: Maximum number of returned functions
    :param int offset: Skip a number of functions
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises UserNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns: requests.Response object
    

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user/{userId}/functions".format(settings["api"]["api_url"], limit=limit, userId=userId, offset=offset, onlyRoles=onlyRoles, keywords=keywords)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        query_parameters.update({ "limit": limit })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        query_parameters.update({ "offset": offset })
    # Only send onlyRoles if the argument was provided, dont send null values
    if onlyRoles is not None:
        query_parameters.update({ "onlyRoles": onlyRoles })
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        query_parameters.update({ "keywords": keywords })

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response
    


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def get_profile(
    userId: int,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Get user profile by ID (PUBLIC)
    
    :param int userId: ID of user
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user/{userId}".format(settings["api"]["api_url"], userId=userId)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def get_user_by_username(
    userName: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Get user profile by username (PUBLIC)
    
    :param str userName: Username
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user/{userName}".format(settings["api"]["api_url"], userName=userName)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def get_user_customers(
    userId: int,
    offset: int = None,
    keywords: str = None,
    keywordField: str = None,
    limit: int = 25,
    keywordMatch: str = "all",
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> Response:
    """List a user's customers. (DEV)
    
    :param int userId: ID of user
    :param int offset: Offset results
    :param list keywords: Search by keywords
    :param list keywordField: Set field strategy for keyword search
    :param int limit: Limit results
    :param str keywordMatch: Set match strategy for keyword search
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises UserNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns: requests.Response object
    

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user/{userId}/customers".format(settings["api"]["api_url"], limit=limit, keywordMatch=keywordMatch, userId=userId, offset=offset, keywords=keywords, keywordField=keywordField)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        query_parameters.update({ "limit": limit })
    # Only send keywordMatch if the argument was provided, dont send null values
    if keywordMatch is not None:
        query_parameters.update({ "keywordMatch": keywordMatch })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        query_parameters.update({ "offset": offset })
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        query_parameters.update({ "keywords": keywords })
    # Only send keywordField if the argument was provided, dont send null values
    if keywordField is not None:
        query_parameters.update({ "keywordField": keywordField })

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response
    


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def get_user_profile_picture(
    user: str,
    domain: str = None,
    default: bool = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> Response:
    """Returns user profile picture. (PUBLIC)
    
    :param str user: User ID or username
    :param str domain: Domain ID or shortname (optional, defaults to current user domain)
    :param bool default: If true, return default avatar image if user picture not set. Default is false.
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns: requests.Response object
    

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user/{user}/picture".format(settings["api"]["api_url"], user=user, domain=domain, default=default)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send domain if the argument was provided, dont send null values
    if domain is not None:
        query_parameters.update({ "domain": domain })
    # Only send default if the argument was provided, dont send null values
    if default is not None:
        query_parameters.update({ "default": default })

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response
    


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def list_users(
    subjectID: int = None,
    offset: int = None,
    limit: int = 25,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Get user profiles (PUBLIC)
    
    :param list subjectID: IDs of users
    :param int offset: Offset results
    :param int limit: Limit results
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user".format(settings["api"]["api_url"], limit=limit, subjectID=subjectID, offset=offset)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        query_parameters.update({ "limit": limit })
    # Only send subjectID if the argument was provided, dont send null values
    if subjectID is not None:
        query_parameters.update({ "subjectID": subjectID })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        query_parameters.update({ "offset": offset })

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def update_profile_picture_for_user_form(
    userID: str,
    domain: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> Response:
    """Upload and modify the user image (PUBLIC)
    
    :param str userID: ID or username of user to change image for
    :param str domain: Domain of the user (defaults to current users domain
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns: requests.Response object
    

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user/{userID}/picture".format(settings["api"]["api_url"], userID=userID, domain=domain)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send domain if the argument was provided, dont send null values
    if domain is not None:
        query_parameters.update({ "domain": domain })

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response
    


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def update_profile_picture_for_user_j_s_o_n(
    userId: str,
    domain: str = None,
    image: str = None,
    mimeType: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> Response:
    """Upload and modify the user image (PUBLIC)
    
    :param str userId: ID or username of user to change image for
    :param str domain: Domain of the user (defaults to current users domain
    :param str image: The image to upload (in base64) 
    :param str mimeType: The MIME type of the image  => Sanitize by regex .+/.+
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns: requests.Response object
    

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user/{userId}/picture".format(settings["api"]["api_url"], userId=userId, domain=domain)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send image if the argument was provided, dont send null values
    if image is not None:
        body.update({ "image": image })
    # Only send mimeType if the argument was provided, dont send null values
    if mimeType is not None:
        body.update({ "mimeType": mimeType })

    query_parameters = {}
    # Only send domain if the argument was provided, dont send null values
    if domain is not None:
        query_parameters.update({ "domain": domain })

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response
    


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def update_user_preferences(
    user: str,
    domain: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Updates users preferences. Body should be a map of preferences to set.Any preference with a null value will be deleted. (DEV)
    
    :param str user: User ID or username
    :param str domain: Domain ID or shortname (optional, defaults to current user domain)
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user/{user}/preferences".format(settings["api"]["api_url"], user=user, domain=domain)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send domain if the argument was provided, dont send null values
    if domain is not None:
        query_parameters.update({ "domain": domain })

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("users","v1","user"),
    module=argus_cli_module
)
def view_user_preferences(
    user: str,
    domain: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """View users preferences. (DEV)
    
    :param str user: User ID or username
    :param str domain: Domain ID or shortname (optional, defaults to current user domain)
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/users/v1/user/{user}/preferences".format(settings["api"]["api_url"], user=user, domain=domain)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send domain if the argument was provided, dont send null values
    if domain is not None:
        query_parameters.update({ "domain": domain })

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()

