"""Autogenerated API"""
import logging
from argus_cli.plugin import register_command
from argus_plugins import argus_cli_module

log = logging.getLogger(__name__)


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def add_vulnerability(
    customer: str = None,
    asset: str = None,
    vulnerability: str = None,
    components: dict = None,
    severity: str = None,
    rawOutput: str = None,
    ttl: int = None,
    cvss: float = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Creates a new vulnerability instance (DEV)
    
    :param str customer: ID or short name of customer, if not specified then current user's customer will be assigned. 
    :param str asset: ID or short name of asset. If short name is specified, then the asset is looked up from the customer. 
    :param str vulnerability: Either vulnerability definition ID (UUID) or vulnerability identifier (e.g. CVE-2015-3429). First the value will be used to lookup vulnerability definition (UUID will only be supported for vulnerability definition lookup). If vulnerability definition exists, then associate this vulnerability instance to it, and require read permission on vulnerability source of the definition. Otherwise create vulnerability instance with vulnerability identifier.  => [\s\w\{\}\$\-\(\)\.\[\]"\'_/\\,\*\+\#:@!?;=]*
    :param list components: Specify components to be added with the vulnerability. 
    :param str severity: Severity level of vulnerability. If not provided, then severity will be calculated based on CVSS value. 
    :param str rawOutput: Raw output from vulnerability scan. Will be sanitized as html. 
    :param int ttl: Time to live duration of this vulnerability since last seen, 0 means won't expire. 
    :param float cvss: CVSS score of vulnerability (range from 0 to 10). (default 0.0)
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import post
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability".format(settings["api"]["api_url"], )
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send customer if the argument was provided, dont send null values
    if customer is not None:
        body.update({ "customer": customer })
    # Only send asset if the argument was provided, dont send null values
    if asset is not None:
        body.update({ "asset": asset })
    # Only send vulnerability if the argument was provided, dont send null values
    if vulnerability is not None:
        body.update({ "vulnerability": vulnerability })
    # Only send components if the argument was provided, dont send null values
    if components is not None:
        body.update({ "components": components })
    # Only send cvss if the argument was provided, dont send null values
    if cvss is not None:
        body.update({ "cvss": cvss })
    # Only send severity if the argument was provided, dont send null values
    if severity is not None:
        body.update({ "severity": severity })
    # Only send rawOutput if the argument was provided, dont send null values
    if rawOutput is not None:
        body.update({ "rawOutput": rawOutput })
    # Only send ttl if the argument was provided, dont send null values
    if ttl is not None:
        body.update({ "ttl": ttl })

    query_parameters = {}

    log.debug("POST %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = post(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def add_vulnerability_definition(
    vulnerabilitySource: str = None,
    vulnerabilityID: str = None,
    name: str = None,
    description: str = None,
    solution: str = None,
    conclusion: str = None,
    references: str = None,
    exploitAvailable: bool = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Creates a new vulnerability definition also require to have read & write permission of vulnerability source that the definition is belonging to. (DEV)
    
    :param str vulnerabilitySource: Define vulnerability source by source ID or short name. 
    :param str vulnerabilityID: Identifier of vulnerability (e.g. plug-in ID from vulnerability scanner). Needs to be unique.  => [\s\w\{\}\$\-\(\)\.\[\]"\'_/\\,\*\+\#:@!?;=]*
    :param str name: Name of vulnerability. Will be sanitized as text. 
    :param str description: Description of vulnerability. Will be sanitized as html. 
    :param str solution: How to fix vulnerability. Will be sanitized as html. 
    :param str conclusion: Short summary of vulnerability. Will be sanitized as html. 
    :param list references: References to vulnerability (e.g. CVE number). Will be sanitized as html. 
    :param bool exploitAvailable: Set if an exploit is available for the vulnerability. 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import post
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/definition".format(settings["api"]["api_url"], )
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send vulnerabilitySource if the argument was provided, dont send null values
    if vulnerabilitySource is not None:
        body.update({ "vulnerabilitySource": vulnerabilitySource })
    # Only send vulnerabilityID if the argument was provided, dont send null values
    if vulnerabilityID is not None:
        body.update({ "vulnerabilityID": vulnerabilityID })
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({ "name": name })
    # Only send description if the argument was provided, dont send null values
    if description is not None:
        body.update({ "description": description })
    # Only send solution if the argument was provided, dont send null values
    if solution is not None:
        body.update({ "solution": solution })
    # Only send conclusion if the argument was provided, dont send null values
    if conclusion is not None:
        body.update({ "conclusion": conclusion })
    # Only send references if the argument was provided, dont send null values
    if references is not None:
        body.update({ "references": references })
    # Only send exploitAvailable if the argument was provided, dont send null values
    if exploitAvailable is not None:
        body.update({ "exploitAvailable": exploitAvailable })

    query_parameters = {}

    log.debug("POST %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = post(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def add_vulnerability_source(
    shortName: str = None,
    name: str = None,
    readPermission: str = None,
    writePermission: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Creates a new VulnerabilitySource (DEV)
    
    :param str shortName: Vulnerability source short name, required and unique  => [a-zA-Z0-9_\-\.]*
    :param str name: Name of source  => [\s\w\{\}\$\-\(\)\.\[\]"\'_/\\,\*\+\#:@!?;=]*
    :param str readPermission: Permission required to allow read access of source & vulnerability definition belong to this source  => [a-zA-Z0-9_\-\.]*
    :param str writePermission: Permission required to allow write access (create/update/delete) source & of vulnerability definition belong to this source  => [a-zA-Z0-9_\-\.]*
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import post
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/source".format(settings["api"]["api_url"], )
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send shortName if the argument was provided, dont send null values
    if shortName is not None:
        body.update({ "shortName": shortName })
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({ "name": name })
    # Only send readPermission if the argument was provided, dont send null values
    if readPermission is not None:
        body.update({ "readPermission": readPermission })
    # Only send writePermission if the argument was provided, dont send null values
    if writePermission is not None:
        body.update({ "writePermission": writePermission })

    query_parameters = {}

    log.debug("POST %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = post(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def delete_vulnerability(
    id: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Delete vulnerability (DEV)
    
    :param str id: vulnerability ID
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import delete
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/{id}".format(settings["api"]["api_url"], id=id)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("DELETE %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = delete(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def delete_vulnerability_definition(
    idOrVulnerabilityID: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Delete Vulnerability definition, also require to have read & write permission of vulnerability source that the definition is belonging to. (DEV)
    
    :param str idOrVulnerabilityID: Vulnerability definition ID or identifier of vulnerability
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import delete
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/definition/{idOrVulnerabilityID}".format(settings["api"]["api_url"], idOrVulnerabilityID=idOrVulnerabilityID)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("DELETE %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = delete(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def delete_vulnerability_source(
    source: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Marks a VulnerabilitySource as deleted (DEV)
    
    :param str source: VulnerabilitySource ID or short name
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import delete
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/source/{source}".format(settings["api"]["api_url"], source=source)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("DELETE %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = delete(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def get_vulnerability_by_id(
    id: str,
    includeRawOutput: bool = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Get Vulnerability by ID. (DEV)
    
    :param str id: Vulnerability ID (UUID)
    :param bool includeRawOutput: Whether include raw output, default not include
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/{id}".format(settings["api"]["api_url"], id=id, includeRawOutput=includeRawOutput)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send includeRawOutput if the argument was provided, dont send null values
    if includeRawOutput is not None:
        query_parameters.update({ "includeRawOutput": includeRawOutput })

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def get_vulnerability_definition(
    idOrVulnerabilityID: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Get Vulnerability definition, also require to have read permission of vulnerability source that the definition is belonging to. (DEV)
    
    :param str idOrVulnerabilityID: Vulnerability definition ID or identifier of vulnerability
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/definition/{idOrVulnerabilityID}".format(settings["api"]["api_url"], idOrVulnerabilityID=idOrVulnerabilityID)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def get_vulnerability_source(
    source: str,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Get VulnerabilitySource (DEV)
    
    :param str source: VulnerabilitySource ID or short name
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/source/{source}".format(settings["api"]["api_url"], source=source)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def resolve_vulnerability(
    id: str,
    resolution: str = None,
    comment: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Resolves a vulnerability. (DEV)
    
    :param str id: Vulnerability ID
    :param str resolution: Specify resolution reason. 
    :param str comment: Comment on why the vulnerability is resolved.  => [\s\w\{\}\$\-\(\)\.\[\]"\'_/\\,\*\+\#:@!?;=]*
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/{id}/resolve".format(settings["api"]["api_url"], id=id)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send resolution if the argument was provided, dont send null values
    if resolution is not None:
        body.update({ "resolution": resolution })
    # Only send comment if the argument was provided, dont send null values
    if comment is not None:
        body.update({ "comment": comment })

    query_parameters = {}

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def search_vulnerability_sources(
    sortBy: str = None,
    source: str = None,
    keywords: str = None,
    keywordFieldStrategy: str = None,
    keywordMatchStrategy: str = None,
    timeFieldStrategy: str = None,
    timeMatchStrategy: str = None,
    subCriteria: dict = None,
    startTimestamp: int = None,
    endTimestamp: int = None,
    includeDeleted: bool = None,
    limit: int = 25,
    offset: int = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Search VulnerabilitySource with specified criteria (DEV)
    
    :param list sortBy: 
    :param list source: Restrict to specified Vulnerability source (ID or short name) 
    :param list keywords: Search for keywords against fields defined by keywordFieldStrategy 
    :param list keywordFieldStrategy: Defines which fields will be searched by keywords (default all supported fields) (default all)
    :param str keywordMatchStrategy: Defines how strict different keywords should be matched (default match all keywords) (default all)
    :param list timeFieldStrategy: Defines which timestamps will be included in the search (default lastUpdatedTimestamp) (default lastUpdatedTimestamp)
    :param str timeMatchStrategy: Defines how strict to match against different timestamps (all/any) using start and end timestamp (default any) (default any)
    :param list subCriteria: Set additional criteria which are applied with logical OR by default 
    :param int startTimestamp: Restrict to a time frame based on the set timeFieldStrategy (start timestamp) (default 0)
    :param int endTimestamp: Restrict to a time frame based on the set timeFieldStrategy (end timestamp) (default 0)
    :param bool includeDeleted: Whether include deleted results (default false)
    :param int limit: Limit maximum amount of results (default 25)
    :param int offset: Skip specified amount of results (default 0)
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import post
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/source/search".format(settings["api"]["api_url"], )
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        body.update({ "sortBy": sortBy })
    # Only send source if the argument was provided, dont send null values
    if source is not None:
        body.update({ "source": source })
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        body.update({ "keywords": keywords })
    # Only send keywordFieldStrategy if the argument was provided, dont send null values
    if keywordFieldStrategy is not None:
        body.update({ "keywordFieldStrategy": keywordFieldStrategy })
    # Only send keywordMatchStrategy if the argument was provided, dont send null values
    if keywordMatchStrategy is not None:
        body.update({ "keywordMatchStrategy": keywordMatchStrategy })
    # Only send startTimestamp if the argument was provided, dont send null values
    if startTimestamp is not None:
        body.update({ "startTimestamp": startTimestamp })
    # Only send endTimestamp if the argument was provided, dont send null values
    if endTimestamp is not None:
        body.update({ "endTimestamp": endTimestamp })
    # Only send timeFieldStrategy if the argument was provided, dont send null values
    if timeFieldStrategy is not None:
        body.update({ "timeFieldStrategy": timeFieldStrategy })
    # Only send timeMatchStrategy if the argument was provided, dont send null values
    if timeMatchStrategy is not None:
        body.update({ "timeMatchStrategy": timeMatchStrategy })
    # Only send subCriteria if the argument was provided, dont send null values
    if subCriteria is not None:
        body.update({ "subCriteria": subCriteria })
    # Only send includeDeleted if the argument was provided, dont send null values
    if includeDeleted is not None:
        body.update({ "includeDeleted": includeDeleted })
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        body.update({ "limit": limit })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        body.update({ "offset": offset })

    query_parameters = {}

    log.debug("POST %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = post(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def search_vulnerability_sources_simplified(
    keywords: str = None,
    keywordField: str = None,
    timeField: str = None,
    source: str = None,
    sortBy: str = None,
    limit: int = 25,
    keywordMatch: str = "all",
    startTimestamp: str = "0",
    endTimestamp: str = "0",
    timeMatch: str = "any",
    offset: int = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Search VulnerabilitySources with specified query parameters (DEV)
    
    :param list keywords: Search by keywords
    :param list keywordField: Set field strategy for keyword search
    :param list timeField: Set field strategy for time range search
    :param list source: Search by sources (ID or short name)
    :param list sortBy: Sort search result
    :param int limit: Maximum number of returned results
    :param str keywordMatch: Set match strategy for keyword search
    :param str startTimestamp: Start timestamp for time range search
    :param str endTimestamp: End timestamp for time range search
    :param str timeMatch: Set match strategy for time range search
    :param int offset: Skip a number of results
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import get
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/source".format(settings["api"]["api_url"], limit=limit, keywordMatch=keywordMatch, startTimestamp=startTimestamp, endTimestamp=endTimestamp, timeMatch=timeMatch, offset=offset, keywords=keywords, keywordField=keywordField, timeField=timeField, source=source, sortBy=sortBy)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}

    query_parameters = {}
    # Only send limit if the argument was provided, dont send null values
    if limit is not None:
        query_parameters.update({ "limit": limit })
    # Only send keywordMatch if the argument was provided, dont send null values
    if keywordMatch is not None:
        query_parameters.update({ "keywordMatch": keywordMatch })
    # Only send startTimestamp if the argument was provided, dont send null values
    if startTimestamp is not None:
        query_parameters.update({ "startTimestamp": startTimestamp })
    # Only send endTimestamp if the argument was provided, dont send null values
    if endTimestamp is not None:
        query_parameters.update({ "endTimestamp": endTimestamp })
    # Only send timeMatch if the argument was provided, dont send null values
    if timeMatch is not None:
        query_parameters.update({ "timeMatch": timeMatch })
    # Only send offset if the argument was provided, dont send null values
    if offset is not None:
        query_parameters.update({ "offset": offset })
    # Only send keywords if the argument was provided, dont send null values
    if keywords is not None:
        query_parameters.update({ "keywords": keywords })
    # Only send keywordField if the argument was provided, dont send null values
    if keywordField is not None:
        query_parameters.update({ "keywordField": keywordField })
    # Only send timeField if the argument was provided, dont send null values
    if timeField is not None:
        query_parameters.update({ "timeField": timeField })
    # Only send source if the argument was provided, dont send null values
    if source is not None:
        query_parameters.update({ "source": source })
    # Only send sortBy if the argument was provided, dont send null values
    if sortBy is not None:
        query_parameters.update({ "sortBy": sortBy })

    log.debug("GET %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = get(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def update_vulnerability(
    id: str,
    vulnerabilityDefinition: str = None,
    addComponents: dict = None,
    deleteComponents: str = None,
    cvss: float = None,
    severity: str = None,
    rawOutput: str = None,
    ttl: int = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Updates an existing Vulnerability (DEV)
    
    :param str id: vulnerability ID
    :param str vulnerabilityDefinition: Assign vulnerability definition. Only allowed when there was no vulnerability definition assigned. 
    :param list addComponents: Specify components to be added with the vulnerability. 
    :param list deleteComponents: Specify IDs of the components to be deleted from the vulnerability. 
    :param float cvss: Update CVSS score of vulnerability (range from 0 to 10). 
    :param str severity: Update severity level of vulnerability. 
    :param str rawOutput: Update raw output from vulnerability scan. Will be sanitized as html. 
    :param int ttl: Time to live duration of this vulnerability since last seen, 0 means won't expire. 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/{id}".format(settings["api"]["api_url"], id=id)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send vulnerabilityDefinition if the argument was provided, dont send null values
    if vulnerabilityDefinition is not None:
        body.update({ "vulnerabilityDefinition": vulnerabilityDefinition })
    # Only send addComponents if the argument was provided, dont send null values
    if addComponents is not None:
        body.update({ "addComponents": addComponents })
    # Only send deleteComponents if the argument was provided, dont send null values
    if deleteComponents is not None:
        body.update({ "deleteComponents": deleteComponents })
    # Only send cvss if the argument was provided, dont send null values
    if cvss is not None:
        body.update({ "cvss": cvss })
    # Only send severity if the argument was provided, dont send null values
    if severity is not None:
        body.update({ "severity": severity })
    # Only send rawOutput if the argument was provided, dont send null values
    if rawOutput is not None:
        body.update({ "rawOutput": rawOutput })
    # Only send ttl if the argument was provided, dont send null values
    if ttl is not None:
        body.update({ "ttl": ttl })

    query_parameters = {}

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def update_vulnerability_definition(
    idOrVulnerabilityID: str,
    name: str = None,
    description: str = None,
    solution: str = None,
    conclusion: str = None,
    addReferences: str = None,
    deleteReferences: str = None,
    exploitAvailable: bool = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Updates an existing Vulnerability definition, also require to have read & write permission of vulnerability source that the definition is belonging to. (DEV)
    
    :param str idOrVulnerabilityID: Vulnerability definition ID or identifier of vulnerability
    :param str name: Update name of the vulnerability definition. Will be sanitized as text. 
    :param str description: Update description of the vulnerability definition. Will be sanitized as html. 
    :param str solution: Update how to fix the vulnerability. Will be sanitized as html. 
    :param str conclusion: Update short summary of the vulnerability. Will be sanitized as html. 
    :param list addReferences: Add references to the vulnerability definition (e.g. CVE number). Will be sanitized as html. 
    :param list deleteReferences: Delete references from the vulnerability definition. Will be sanitized as html first before check against existing references. 
    :param bool exploitAvailable: Update whether an exploit is available for the vulnerability. 
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/definition/{idOrVulnerabilityID}".format(settings["api"]["api_url"], idOrVulnerabilityID=idOrVulnerabilityID)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({ "name": name })
    # Only send description if the argument was provided, dont send null values
    if description is not None:
        body.update({ "description": description })
    # Only send solution if the argument was provided, dont send null values
    if solution is not None:
        body.update({ "solution": solution })
    # Only send conclusion if the argument was provided, dont send null values
    if conclusion is not None:
        body.update({ "conclusion": conclusion })
    # Only send addReferences if the argument was provided, dont send null values
    if addReferences is not None:
        body.update({ "addReferences": addReferences })
    # Only send deleteReferences if the argument was provided, dont send null values
    if deleteReferences is not None:
        body.update({ "deleteReferences": deleteReferences })
    # Only send exploitAvailable if the argument was provided, dont send null values
    if exploitAvailable is not None:
        body.update({ "exploitAvailable": exploitAvailable })

    query_parameters = {}

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()


@register_command(
    extending=("assets","v2","vulnerability"),
    module=argus_cli_module
)
def update_vulnerability_source(
    source: str,
    shortName: str = None,
    name: str = None,
    readPermission: str = None,
    writePermission: str = None,
    json: bool = True,
    verify: bool = True,
    apiKey: str = None,
    authentication: dict = {}
  ) -> dict:
    """Updates an existing VulnerabilitySource (DEV)
    
    :param str source: VulnerabilitySource ID or short name
    :param str shortName: Update short name of vulnerability source  => [a-zA-Z0-9_\-\.]*
    :param str name: Update name of vulnerability source  => [\s\w\{\}\$\-\(\)\.\[\]"\'_/\\,\*\+\#:@!?;=]*
    :param str readPermission: Update the permission required to allow read access of source & vulnerability definition belong to the source  => [a-zA-Z0-9_\-\.]*
    :param str writePermission: Update the permission required to allow write access (create/update/delete) of source & vulnerability definition belong to the source  => [a-zA-Z0-9_\-\.]*
    :raises AuthenticationFailedException: on 401
    :raises AccessDeniedException: on 403
    :raises ObjectNotFoundException: on 404
    :raises ValidationErrorException: on 412:raises ArgusException: on other status codes
    
    :returns dictionary translated from JSON

    """
    from os import getenv
    from requests import put
    from argus_api._validators import validate_http_response
    
    from argus_cli.settings import settings
    
    url = "{}/assets/v2/vulnerability/source/{source}".format(settings["api"]["api_url"], source=source)
    headers = {
        'User-Agent': 'ArgusToolbelt/',
        'content': 'application/json'
    }

    if not apiKey and "api_key" in settings["api"]:
        apiKey = settings["api"]["api_key"]

    if apiKey:
        headers["Argus-API-Key"] = apiKey
    elif authentication and isinstance(authentication, dict):
        headers.update(authentication)
    elif callable(authentication):
        headers.update(authentication(url))

    body = {}
    # Only send shortName if the argument was provided, dont send null values
    if shortName is not None:
        body.update({ "shortName": shortName })
    # Only send name if the argument was provided, dont send null values
    if name is not None:
        body.update({ "name": name })
    # Only send readPermission if the argument was provided, dont send null values
    if readPermission is not None:
        body.update({ "readPermission": readPermission })
    # Only send writePermission if the argument was provided, dont send null values
    if writePermission is not None:
        body.update({ "writePermission": writePermission })

    query_parameters = {}

    log.debug("PUT %s (headers: %s, body: %s)" % (url, str(headers), str(body) or ""))

    response = put(
        url,
        params=query_parameters or None,
        json=body or None,
        verify=getenv('REQUESTS_CA_BUNDLE', verify),
        headers=headers
    )

    validate_http_response(response)
    return response.json()

